package com.bawei.mall.car.mapper;

import com.bawei.mall.car.domain.Car;
import com.bawei.mall.car.domain.CarBySku;
import com.bawei.mall.car.domain.ProductSkuInfo;
import com.bawei.mall.product.domain.MallProductSkuInfo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @program: mall_cloud
 * @ClassName: CarMapper
 * @author: Gyc
 * @create: 2024-05-06 16:03
 **/
@Mapper
public interface CarMapper {


    String queryCar(Long userId);


    void insertCar(@Param("userId") Long userId, @Param("carId") String carId);

    void addCarBySkuId(@Param("skuId") Integer skuId, @Param("carId") String carId, @Param("userId") Long userId);

    CarBySku showCarBySkuId(@Param("skuId") Integer skuId, @Param("carId") String carId);

    void updCarBySkuId(@Param("skuId") Integer skuId, @Param("carId") String carId, @Param("i") int i);

    void delCarBySkuId(String carId);

    void delCar(Long userId);

    void delCarBySkuIdInfo(@Param("skuId") Integer skuId, @Param("carId") String carId);
    //查询这个用户所有的skuId
    List<Integer> querySkuId(String carId);

    MallProductSkuInfo querySku(Integer skuId);
    //查询所有的sku和商品表
    List<ProductSkuInfo> querySkus(@Param("skuIds") List<Integer> skuIds, @Param("carId") String carId);

    List<ProductSkuInfo> ProductBySkuIdList();

    Car queryCarByCarId(String carId);
}
